//=======================================================================
// main.cpp : Example -- how to use EMPlane (EMTV)
//
// Copyright (c) 2009 Tai-Pang Wu (pang@cse.ust.hk) and HKUST x CUHK
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

#include "StdAfx.h"
#include "emtv.h"

#include <stdio.h>
#include <stdlib.h>

#define  MAX_CHAR_LENGTH     1024

int main(int argc, char *argv[])
{	
	int i; 

	// read data from file
	// fileName       : file path
	// numPt[output]  : number of input points
	// dim            : dimension of the point
	char fileName[MAX_CHAR_LENGTH] = "temp0.0200.txt";
	int  numPt = 0;
	int  dim = 0;
	double** x = readFromFile( fileName, numPt, dim );


	// create EMPlane object
	// dim           : the dimension of the input
	// sigma_scale   : the scale parameter for voting (\sigma_d of Eqn 2 (or Eqn 4 in reference [28]))
	// sigma0        : the lower bound of \sigma of Eqn 5 (or Eqn 23 in reference [28])
	// sigma1        : the lower bound of \sigma1 of Eqn 5 (or Eqn 23 in reference [28])
	// sigma2        : the lower bound of \sigma2 of Eqn 5 (or Eqn 23 in reference [28])
	// maxIter       : maximum number of iteration (1 iteration = one complete E- and M-Step)
	// epsBoundary   : the cutoff boundary for Eqn 2. Note that the tail of Gaussian is long where 
	//                 the values smaller than epsBoundary will be ignored.
	double sigma_scale  = 0.01;
	double sigma0       = EPLSION;
	double sigma1       = EPLSION;
	double sigma2       = EPLSION;
	int    maxIter      = 400;
	double epsBoundary  = 0.4;
	EMPlane emp( dim, sigma_scale, sigma0, sigma1, sigma2, maxIter, epsBoundary);


	// Perform EM plane fitting
	// x             : the set of input points
	// numPt         : number of input points
	// v[output]     : the output model (see Eqn 13 of reference [28])
	// K[output]     : the set of optimized tensors correspond to the input points x
	// w[output]     : the weights correspond to the input points x (i.e. Eqn 4 (or Eqn 19 in reference [28]))
	// return        : number of interation used to obtain the outputs
	double* v     = NULL;
	double*** K   = NULL;
	double* w     = NULL;
	int iter_em = emp.em_plan( x, numPt, v, K, w ); 


	// Note that the ground truth for this example is v = [ 0.71, -0.71 ] (or [ -0.71, 0.71 ]).
	printf( "Estimated plane = [ " );
	for( i = 0; i < dim - 1; i++ )
		printf( "%0.2f, ", v[i] );
	printf( "%0.2f ]\n", v[dim-1] );


	// De-allocate memory
	safeDelete2DDouble( numPt, x );
	SAFEDELARR( v );
	SAFEDELARR( w );
	if( K != NULL )
	{
		for( i = 0; i < numPt; i++ )
			safeDelete2DDouble( dim, K[i] );
		SAFEDELARR( K );
	}
	
	return 0;
}
